@@ -74,7 +74,8 @@ DBG (const char *str, xexp *cat)
 }
 #endif
 
-bool verbose;
+bool verbose = false;
+bool prepend = false;
 
 xexp *catalogues;
 xexp *domains;
@@ -113,6 +114,7 @@ update_conf ()
 {
   if (!write_sources_list (CATALOGUE_APT_SOURCE, catalogues))
     exit (1);
+  backup_catalogues ();
 }
 
 xexp *
@@ -128,10 +130,10 @@ find_element (xexp *conf, xexp *elt,
 const char *
 element_description (xexp *element)
 {
-  const char *desc = xexp_aref_text (element, "name");
-  if (desc == NULL)
-    desc = "<unnamed>";
-  return desc;
+  xexp *desc = xexp_aref (element, "name");
+  if (xexp_is_text (desc))
+    return xexp_text (desc);
+  return xexp_aref_text (desc, "default");
 }
 
 void
@@ -144,19 +146,24 @@ handle_generic_element (xexp *conf, xexp *element,
   if (verbose)
     {
       if (old_element && add)
-	printf ("Replacing %s\n", element_description (element));
+	printf ("Replacing '%s'\n", element_description (element));
       else if(old_element == NULL && add)
-	printf ("Adding %s\n", element_description (element));
+	printf ("Adding '%s'\n", element_description (element));
       else if (old_element && !add)
-	printf ("Removing %s\n", element_description (element));
+	printf ("Removing '%s'\n", element_description (element));
       else
-	printf ("Not found: %s\n", element_description (element));
+	printf ("Not found: '%s'\n", element_description (element));
     }
 
   if (old_element)
     xexp_del (conf, old_element);
   if (add)
-    xexp_append_1 (conf, element);
+    {
+      if (prepend)
+	xexp_cons (conf, element);
+      else
+	xexp_append_1 (conf, element);
+    }
   else
     xexp_free (element);
 }
@@ -248,6 +255,13 @@ main (int argc, char **argv)
       argv++;
     }
 
+  if (argc > 1 && strcmp (argv[1], "--prepend") == 0)
+    {
+      prepend = true;
+      argc--;
+      argv++;
+    }
+
   if (argc < 2)
     usage ();
 
