@@ -56,7 +56,6 @@
 extern "C" {
   #include <hildon/hildon-window.h>
   #include <hildon/hildon-note.h>
-  #include <libosso.h>
   #include <hildon/hildon-help.h>
   #include <hildon/hildon-bread-crumb-trail.h>
 }
@@ -1444,7 +1443,7 @@ rpc_show_error_report (void *data)
 	     msg,
 	     _("ai_bd_ok"), GTK_RESPONSE_OK,
 	     _("ai_ni_bd_details"), 1,
-	     _("ai_bd_notice_cancel"), GTK_RESPONSE_CANCEL,
+	     _("ai_bd_cancel"), GTK_RESPONSE_CANCEL,
 	     NULL);
 
 	  g_free (msg);
@@ -1462,8 +1461,8 @@ rpc_show_error_report (void *data)
       dialog = hildon_note_new_confirmation_add_buttons 
 	(NULL,
 	 _("ai_ni_update_list_not_successful"),
-	 _("ai_ni_bd_retry"), 2,
-	 _("ai_bd_notice_cancel"), GTK_RESPONSE_CANCEL,
+	 _("ai_bd_ok"), 2,
+	 _("ai_bd_cancel"), GTK_RESPONSE_CANCEL,
 	 NULL);
     }
   
@@ -2185,8 +2184,12 @@ install_named_package (int state, const char *package,
   c->data = data;
 
   if (p == NULL)
-    annoy_user (_("ai_ni_error_download_missing"),
-		inp_no_package, c);
+    {
+      char *text = g_strdup_printf (_("ai_ni_error_download_missing"),
+				    package);
+      annoy_user (text, inp_no_package, c);
+      g_free (text);
+    }
   else
     {
       package_info *pi = (package_info *) p->data;
@@ -2856,6 +2859,12 @@ with_initialized_packages (void (*cont) (void *data), void *data)
     }
 }
 
+osso_context_t *
+get_osso_context ()
+{
+  return osso_ctxt;
+}
+
 int
 main (int argc, char **argv)
 {
